/* $Id$
 *
 * ebp2prom - Extended Behavior Protocols translator to Promela
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */



package org.ow2.dsrg.fm.ebp2promela;

import java.io.*;
import org.ow2.dsrg.fm.bptools.EBPEventTableImpl;
import org.ow2.dsrg.fm.bptools.Preferences;
import org.ow2.dsrg.fm.ebp2promela.generator.EmbeddingGenerator;
import org.ow2.dsrg.fm.bptools.architecture.ParsedArchitecture;
import org.ow2.dsrg.fm.bptools.architecture.ParsedArchitecture.ArchitectureException;
import org.ow2.dsrg.fm.bptools.architecture.ParsedArchitecture.ParserBridge;
import org.ow2.dsrg.fm.bptools.architecture.PreprocessedArchitecture;
import org.ow2.dsrg.fm.bptools.javacc.archparser.ArchitectureParser;
import org.ow2.dsrg.fm.bptools.javacc.ebpparser.EBPParser;
import org.ow2.dsrg.fm.bptools.parser.nodes.parsed.BPParsedComponent;


/**
 * Main class for testing purposes.
 *
 * @author Kofron
 */
public class EBPArch2Promela {
    
    public static class EBPParserBridge implements ParserBridge{
        public ParsedArchitecture parseArchitecture(Reader in) throws ArchitectureException {
            assert false;
            return null;
        }

        public BPParsedComponent parseComponent(Reader in) throws ArchitectureException {
            
        try{
            return new EBPParser(in).parse();
        } catch (org.ow2.dsrg.fm.bptools.javacc.ebpparser.ParseException pe){
            throw new ParsedArchitecture.ParseException(pe.getMessage());
        }
        
        }

    }
    
    
    public static void main(String[] args) throws ArchitectureException {
        
        EBPEventTableImpl et = new EBPEventTableImpl();
        EmbeddingGenerator gen = new EmbeddingGenerator(et);
        
        try {
            Preferences.setBpVersion(Preferences.BPVersion.EXTENDED);
            
            File fi = new File(args[0]);
            String pathPart =  fi.getParent();
            if (pathPart==null){
                pathPart = "."+File.separator;
            }
            ArchitectureParser p = new ArchitectureParser(new FileReader(fi), pathPart);
            ParsedArchitecture pa = p.architecture(new EBPParserBridge() );    
            PreprocessedArchitecture arch = new PreprocessedArchitecture(pa,et);
            //new PrintStream(new FileOutputStream("preprocessedArch.bp")).print(arch.print());

            String dirname;
            if (args.length==2){
                dirname=args[1];
                File f = new File(dirname);
                if (!f.exists()){
                    f.mkdir();
                }
                gen.generate(arch, dirname);
            }else{
                System.out.println("Specify the directory name");
            }

        } catch (org.ow2.dsrg.fm.bptools.javacc.archparser.ParseException e) {
            System.out.println("Parse error: " + e.toString());
        } catch (FileNotFoundException e) {
            System.out.println("The input file not found, sorry. "+ e.getMessage());
        } catch (IOException e) {
            System.out.println("Problems accessing the input/output files.");
        }
    }
    
}
