/* $Id: EBPCreateUndetFSMActions.java 90 2007-11-16 11:54:37Z poch $
 *
 * ebp2prom - Extended Behavior Protocols translator to Promela
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.ow2.dsrg.fm.ebp2promela.generator;


import java.util.LinkedList;
import java.util.List;
import org.ow2.dsrg.fm.bptools.EBPEventTableImpl;
import org.ow2.dsrg.fm.bptools.IEventTable;
import org.ow2.dsrg.fm.bptools.fsm.EBPNondeterministicFSM;
import org.ow2.dsrg.fm.bptools.fsm.NondeterministicFSM;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPAcceptRq;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPAssign;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPEmitRq;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPNode;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPNull;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPSwitch;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPSynchro;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPWhile;
import org.ow2.dsrg.fm.bptools.parser.traversal.CreateUndetFSMActions;


/**
 *
 * @author Kofron
 */
public class EBPCreateUndetFSMActions extends CreateUndetFSMActions {
    
    
    /** Creates a new instance of EBPCreateUndetFSMActions */
    public EBPCreateUndetFSMActions(IEventTable et) {
        super(et);
        
    }
    
    
    public NondeterministicFSM whileAction(BPWhile node) {
        EBPNondeterministicFSM child = (EBPNondeterministicFSM)action(node.child());
        
        child.whileNode(node);
        
        return child;
    }
    
    public NondeterministicFSM switchAction(BPSwitch node) {
        List<NondeterministicFSM> children = new LinkedList<NondeterministicFSM>();
        EBPNondeterministicFSM result = (EBPNondeterministicFSM)newEmptyFSM();
        
        for (BPNode n: node.getChildren()){
            NondeterministicFSM child = action(n);
            if (child.isNullReducible()){
                EBPNondeterministicFSM tmp = (EBPNondeterministicFSM)newEmptyFSM();
                tmp.tau(new BPNull(eventTable.getLambdaEvent()));
                tmp.concat(child);
                child = tmp;
            }
            children.add(child);
        }
        
        BPNode defaultBranchParseTree = node.getDefaultBranch();
        NondeterministicFSM defaultBranch = null;
        if (defaultBranchParseTree!=null){
            defaultBranch= (NondeterministicFSM) defaultBranchParseTree.apply(this);
            if (defaultBranch.isNullReducible()){
                EBPNondeterministicFSM n = (EBPNondeterministicFSM)newEmptyFSM();
                n.tau(new BPNull(eventTable.getLambdaEvent()));
                n.concat(defaultBranch);
                defaultBranch = n;
                
            }
        }
        
        
        return result.switchNode(node, children, defaultBranch);
        
    }
    
    
    public NondeterministicFSM synchroAction(BPSynchro node) {
        NondeterministicFSM result = newEmptyFSM();
        
        result.atomic(node);
        return result;
    }
    
    
    
    public NondeterministicFSM assignAction(BPAssign node) {
        NondeterministicFSM result = newEmptyFSM();
        
        result.atomic(node);
        return result;
    }
    
    
    public NondeterministicFSM emitRqAction(BPEmitRq node) {
        NondeterministicFSM result = newEmptyFSM();
        EBPEventTableImpl eet = (EBPEventTableImpl)eventTable;
        
        result.atomic(node);
        return result;
    }
    
    public NondeterministicFSM acceptRqAction(BPAcceptRq node) {
        NondeterministicFSM result = newEmptyFSM();
        
        EBPEventTableImpl eet = (EBPEventTableImpl)eventTable;
        
        result.atomic(node);
        return result;
    }
    
}
