/* $Id$
 *
 * ebp2prom - Extended Behavior Protocols translator to Promela
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.ow2.dsrg.fm.ebp2promela.traversal;

import java.io.PrintStream;
import org.ow2.dsrg.fm.bptools.parser.nodes.AndGuard;
import org.ow2.dsrg.fm.bptools.parser.nodes.ComparisonGuard;
import org.ow2.dsrg.fm.bptools.parser.nodes.Guard;
import org.ow2.dsrg.fm.bptools.parser.nodes.NotInGuard;
import org.ow2.dsrg.fm.bptools.parser.nodes.TrueGuard;
import org.ow2.dsrg.fm.bptools.parser.traversal.GuardActions;

/**
 * Translates Guard parse-tree into Promela (in fact a C function).
 *
 * @author Ondrej Sery
 */
public class Guard2PromelaActions extends GuardActions<Void> {
    
    /** Stream for Promela output. */
    private PrintStream output;
    /**
     * Id of the promela process representing the component for which the
     * guards are being generated.
     */
    private String componentId;
    
    /** Creates a new instance of Guard2PromelaActions */
    public Guard2PromelaActions() {
    }
    
    /**
     * Generates the Promela code.
     *
     * @param output Stream for Promela output.
     * @param guard Guard to be translated.
     * @param guardId String identification of the guard.
     * @param componentId Strinbg id of the component (Promela process) for
     *          which the guard is being generated
     */
    public void generateGuard(PrintStream output, Guard guard, String guardId,
            String componentId) {
        this.output = output;
        this.componentId = componentId;
        
        if (!TrueGuard.instance().equals(guard)) {
            output.println("int " + guardId + " () {");
            output.print("\treturn ");
            guard.apply(this);
            output.println(";\n}");
        }
    }
    
    /** Action for a TrueGuard */
    public Void trueAction(TrueGuard node) {
        // NOTE: We should not get here.
        assert false;
        return null;
    }
    
    /** Action for a AndGuard */
    public Void andAction(AndGuard node) {
        output.print("(");
        node.getLeft().apply(this);
        output.print(") && (");
        node.getRight().apply(this);
        output.print(")");
        return null;
    }
    
    /** Action for a ComparisonGuard */
    public Void comparisonAction(ComparisonGuard node) {
        output.print("V" + componentId + "_" + node.getVariable() + " == " +
                node.getValue());
        
        return null;
    }
    
    public Void notInAction(NotInGuard node) {
        output.print("(");
        boolean first = true;
        
        for (String val : node.getValue()) {
            if (first) first = false;
            else output.print("&&");
            output.print("(");
            output.print("V" + componentId + "_" +node.getVariable() + "!=" 
                    + val);
            output.print(")");
        }
        
        output.print(")");
        return null;
    }
}
