/* $Id$
 *
 * ebp2prom - Extended Behavior Protocols translator to Promela
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.fm.ebp2promela.traversal;

import java.util.HashMap;
import java.util.Map;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPAccept;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPAcceptResp;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPAcceptRq;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPAcceptSubprot;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPAlternative;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPAndParallel;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPAssign;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPEmit;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPEmitResp;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPEmitRq;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPEmitSubprot;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPNode;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPNull;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPOrParallel;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPParentheses;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPRepetition;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPSequence;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPSwitch;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPSynchro;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPWhile;
import org.ow2.dsrg.fm.bptools.parser.traversal.BPTreeActions;

/**
 * Analysis of multi-synchronisation action. The goal is to collect information
 * about synchro event usage among processes.
 * 
 * @author Ondrej Sery
 */
public class MultiSyncAnalysisActions extends BPTreeActions<Void> {

    /**
     * Id of the promela process representing the component for which the
     * guards are being generated.
     */
    private String componentId;
    
    /**
     * EventId -> ComponentId -> index.
     */
    private Map<Integer,Map<String,Integer>> result =
            new HashMap<Integer,Map<String,Integer>>();

    /** Creates a new instance of MultiSyncAnalysisActions */
    public MultiSyncAnalysisActions() {
    }
    
    public Map<Integer,Map<String,Integer>> getResult() {
        return result;
    }
    
    /**
     * Generates the Promela code.
     *
     * @param node Root node of the BP to be analyzed
     * @param componentId Strinbg id of the component (Promela process) for
     *          which the guard is being generated
     */
    public void analyzeMultiSync(BPNode node, String componentId) {
        this.componentId = componentId;
        
        action(node);
    }
    
    //switch
    public Void switchAction(BPSwitch node) {
        for (BPNode n : node.getChildren())
            n.apply(this);
        return null;
    }
    
    // binary classes

    public Void andParallelAction(BPAndParallel node) {
        node.left().apply(this);
        node.right().apply(this);
        return null;
    }

    public Void orParallelAction(BPOrParallel node) {
        node.left().apply(this);
        node.right().apply(this);
        return null;
    }

    public Void alternativeAction(BPAlternative node) {
        node.left().apply(this);
        node.right().apply(this);
        return null;
    }
    
    public Void sequenceAction(BPSequence node) {
        node.left().apply(this);
        node.right().apply(this);
        return null;
    }
    
    // unary classes

    public Void repetitionAction(BPRepetition node) {
        node.child().apply(this);
        return null;
    }
    
    public Void parenthesesAction(BPParentheses node) {
        node.child().apply(this);
        return null;
    }
    
    public Void acceptSubprotAction(BPAcceptSubprot node) {
        node.child().apply(this);
        return null;
    }

    public Void emitSubprotAction(BPEmitSubprot node) {
        node.child().apply(this);
        return null;
    }
    
    public Void whileAction(BPWhile node) {
        node.child().apply(this);
        return null;
    }
    
    // leaves

    public Void emitRqAction(BPEmitRq node) {
        return null;
    }

    public Void acceptRqAction(BPAcceptRq node) {
        return null;
    }

    public Void emitRespAction(BPEmitResp node) {
        return null;
    }

    public Void acceptRespAction(BPAcceptResp node) {
        return null;
    }

    public Void emitAction(BPEmit node) {
        return null;
    }

    public Void acceptAction(BPAccept node) {
        return null;
    }

    public Void nullAction(BPNull node) {
        return null;
    }
    
    public Void synchroAction(BPSynchro node) {
        int ev = node.eventCode();
        Map<String, Integer> evMap = result.get(ev);
        
        if (evMap == null) {
            evMap = new HashMap<String, Integer>();
            result.put(ev, evMap);
        } else if (evMap.containsKey(componentId)) {
            // Already there
            return null;
        }
        
        evMap.put(componentId, evMap.size());
        return null;
    }
    
    public Void assignAction(BPAssign node) {
        return null;
    }
}
