/* $Id: Generator.java 85 2007-10-03 15:07:46Z poch $
 *
 * ebp2prom - Extended Behavior Protocols translator to Promela
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.ow2.dsrg.fm.ebp2promela.generator;

import java.io.*;
import java.util.*;
import java.util.logging.Logger;
import org.ow2.dsrg.fm.bptools.IEventTable;
import org.ow2.dsrg.fm.bptools.ProtocolKeeper;
import org.ow2.dsrg.fm.bptools.architecture.PreprocessedArchitecture;
import org.ow2.dsrg.fm.bptools.fsm.CompactFSM;
import org.ow2.dsrg.fm.bptools.fsm.SimpleState;
import org.ow2.dsrg.fm.bptools.fsm.State;
import org.ow2.dsrg.fm.bptools.parser.nodes.BPComponent;

/**
 * This class exploits the generated javacc parser and dchecker functionality
 * to take a behavior specification in extended behavior protocols and
 * generates somewhat optimized finite automata and data structure in
 * Promela format. This generator is deprecated by EmbeddingGenerator.
 *
 * @author Kofron
 * @deprecated
 */
public class Generator {
    
    private static String STATE_LABEL_PREFIX = "_";
    private static String EVENT_REQUEST_PREFIX = "S";
    private static String EVENT_RESPONSE_PREFIX = "E";
    private static String EVENT_BIT_VAR_PREFIX = "";
    private static String GLOBAL_PROCESS_COUNTER = "pcount";
    private static String FINAL_STATE_SIGNAL = "finish";
    private static String EXIT_STATE_LABEL = "DONE";
    
    /** Creates a new instance of Generator */
    public Generator() {
    }
    
    
    /**
     * Generates output in Promela.
     *
     * @param architecture Preprocessed architecture for Promela generation.
     * @param path Path for generation of files (null for working directory).
     *
     * @throws IOException On problems accessing the output files.
     */
    public void generate(PreprocessedArchitecture architecture, String path)
    throws IOException {
        if (path == null) {
            path = "";
        } else if (path.length() > 0 && !path.endsWith(File.separator)) {
            path = path.concat(File.separator);
        }
        
        et = architecture.getEventTable();
        
        // TODO: By ignoring position of "sync"s among protocols, we slightly
        // change semantic of consent as if the top level consent (from
        // the consent tree) is synced by a union of individual syncs, while
        // other consents are synced by an empty set.
        
        prepareAutomata(architecture);
        
        List<BPComponent> components = architecture.getComponents();
        int i = 0;
        // TODO: Iterate through the protocols and syncs and generate one
        // component_name.pr file per component
        for (BPComponent component : components) {
            // FIXME: dump a file with the automaton
            /*
            try {
                protocol.getFlyFSM().dump(
                        new PrintStream(path + "automaton_" + i + ".dt"));
            } catch (FileNotFoundException e) {
                System.out.println("Could not dump automaton, sorry.");
            }*/
            
            PrintStream pout = new PrintStream(path + "component_" + i + ".pr");
            generatePreamble(pout, component.getProtocolKeeper());
            generateProcess(pout, component.getProtocolKeeper(), 
                    String.valueOf(i));
            pout.close();
            i++;
        }
        
        generateMain(architecture, path);
        generateGlobals(architecture, path);
    }
    
    /**
     * Generates Promela preamble to the output stream. The preamble contains
     * all necessary macros, includes, and so on. The rest is generated using
     * generateProcess().
     */
    private void generatePreamble(PrintStream output, ProtocolKeeper protocol) {
        output.println("#include \"stdebp.pr\"");
        output.println("#include \"global.pr\"");
        output.println();
    }
    
    /**
     * Generates Promela main in file path + main.pr.
     *
     * @throws IOException on output file problems
     */
    private void generateMain(PreprocessedArchitecture architecture,
            String path) throws IOException {
        PrintStream pout = new PrintStream(path + "main.pr");
        List<BPComponent> components = architecture.getComponents();
        
        pout.println("// Standard macros for BP event synchronization.");
        pout.println("#include \"stdebp.pr\"");
        pout.println("// Definitions shared by all components.");
        pout.println();
        
        pout.println("// All components.");
        for (int i = 0; i < components.size(); i++) {
            pout.println("#include \"component_" + i + ".pr\"");
        }
        pout.println();
        
        pout.println(
                "// Main process\n" +
                "init {\n" +
                "\tatomic {\n" +
                "\t\tpcount = " + components.size() + ";"
                );
        for (int i = 0; i < components.size(); i++) {
            pout.println("\t\trun component_" + i + "();");
        }
        pout.println("\t}");
        pout.println();
        
        pout.println("\t// Verification stopped. It can be due to \n" +
                "\t// 1) Successfull verification\n" +
                "\t// 2) Bad activity\n" +
                "\t// 3) No activity");
        pout.println("\ttimeout;");
        pout.println("\td_step {\n" +
                "\t\tif\n" +
                "\t\t:: bad_activity ->\n" +
                "\t\t\tprintf(\"BAD ACTIVITY\\n\");\n" +
                "\t\t\tassert(0);\n" +
                "\t\t:: else -> skip\n" +
                "\t\tfi;\n" +
                "\t\tEOR = 1;\n" +
                "\t}");
        pout.println();
        
        pout.println("\ttimeout;\n" +
                "\td_step {\n" +
                "\t\tif\n" +
                "\t\t:: pcount > 0 ->\n" +
                "\t\t\tprintf(\"NO ACTIVITY\\n\");\n" +
                "\t\t\tassert(0);\n" +
                "\t\t:: else -> skip\n" +
                "\t\tfi;\n" +
                "\t}\n" +
                "}");
        
        pout.close();
    }
    
    /**
     * Generates Promela global shared definitions in file path + global.pr.
     *
     * @throws IOException on output file problems
     */
    private void generateGlobals(PreprocessedArchitecture architecture,
            String path) throws IOException {
        
        /* NOTE: The commented code was unable to compile, since the Generator 
                    class is deprecated, the code was only commented out.
        
        PrintStream pout = new PrintStream(path + "global.pr");
        
        pout.println("#ifndef GLOBAL_PR");
        pout.println("#define GLOBAL_PR");
        pout.println();
        
        List<Set<Integer>> syncs = architecture.getSyncs();
        Set<Integer> synced = new HashSet<Integer>();
        
        for (Set<Integer> sync : syncs) {
            for (Integer event : sync) {
                if (!synced.contains(event)) {
                    synced.add(event);
                    pout.println("bit " + eventToBitVar(
                            et.getEventCode(event, et.acceptRespEvent)) + ";");
                    pout.println("bit " + eventToBitVar(
                            et.getEventCode(event, et.acceptRqEvent)) + ";");
                }
            }
        }
        
        Set<Integer> unbound = architecture.getUnbound();
        
        if (unbound != null) {
            for (Integer event : unbound) {
                if (!synced.contains(event)) {
                    synced.add(event);
                    pout.println("bit " + eventToBitVar(
                            et.getEventCode(event, et.acceptRespEvent)) + ";");
                    pout.println("bit " + eventToBitVar(
                            et.getEventCode(event, et.acceptRqEvent)) + ";");
                }
            }
        }
        
        pout.println();
        pout.println("#endif");
        
        pout.close();
         */
    }
    
    /**
     * Generates Promela process to the output stream. Preamble using
     * generatePreamble() should be created before the process.
     */
    private void generateProcess(PrintStream output, ProtocolKeeper protocol,
            String id) {
        output.println("proctype component_" + id + "()");
        output.println("{");
        
        CompactFSM cfsm = protocol.getCompactFSM();
        SimpleState[] states = cfsm.getStateTable();
        int startState = cfsm.getStartState();
        
        // Print the initial state first, so that we save one transition
        // in Promela ;o)
        generateState(output, protocol, startState, states);
        
        for (int i = 0; i < states.length; i++) {
            if (i != startState)
                generateState(output, protocol, i, states);
        }
        
        output.println(EXIT_STATE_LABEL + ":");
        output.println("skip;");
        output.println("}");
    }
    
    /**
     * Generates a single state of the automaton (that is label and transitions
     * using if and other macros).
     */
    private void generateState(PrintStream output, ProtocolKeeper protocol,
            int state, SimpleState[] stateTable) {
        output.println(stateToLabel(state) + ":");
        output.println("if");
        
        SimpleState s = stateTable[state];
        int e = s.nextEdge(State.BEFORE_FIRST_EDGE);
        
        while (e != State.AFTER_LAST_EDGE) {
            if (et.isAccept(e)) {
                generateAcceptTransition(output, e, s.getEdgeTarget(e));
            } else if (et.isEmit(e)) {
                generateEmitTransition(output, e, s.getEdgeTarget(e));
            } else {
                // We can only accept or emit at the time...
                assert false;
            }
            e = s.nextEdge(e);
        }
        
        if (s.isFinal()) {
            generateFinalState(output);
        }
        
        output.println("fi;");
    }
    
    /**
     * Generates emit transition identified by the given event.
     */
    private void generateEmitTransition(PrintStream output,
            int event, int nextState) {
        output.println("::" + "emit(" +
                eventToBitVar(event) + "," + stateToLabel(nextState) +
                ");");
    }
    
    /**
     * Generates accept transition identified by the given event.
     */
    private void generateAcceptTransition(PrintStream output,
            int event, int nextState) {
        output.println("::" + "accept(" +
                eventToBitVar(event) + "," + stateToLabel(nextState) +
                ");");
    }
    
    /**
     * Generates accept transition identified by the given event.
     */
    private void generateFinalState(PrintStream output) {
        // TODO: Generate final state transition
        output.println("::" + FINAL_STATE_SIGNAL + "(" + 
                EXIT_STATE_LABEL + ");");
    }
    
    /**
     * Converts an event identification to String representing its Promela
     * bit variable.
     */
    private String eventToBitVar(int event) {
        /* NOTE: Too big output...
        String method = et.getString(et.getMethodCode(event));
        method = method.replace('.','_');
        
        if (et.isAcceptResp(event)) {
            return EVENT_BIT_VAR_PREFIX + method + EVENT_RESPONSE_SUFFIX;
        } else if (et.isAcceptRq(event)) {
            return EVENT_BIT_VAR_PREFIX + method + EVENT_REQUEST_SUFFIX;
        } else if (et.isEmitResp(event)) {
            return EVENT_BIT_VAR_PREFIX + method + EVENT_RESPONSE_SUFFIX;
        } else if (et.isEmitRq(event)) {
            return EVENT_BIT_VAR_PREFIX + method + EVENT_REQUEST_SUFFIX;
        }
        assert false;
        return null;
         */
        if (et.isResp(event)) {
            return EVENT_BIT_VAR_PREFIX + EVENT_RESPONSE_PREFIX + event;
        } else if (et.isRq(event)) {
            return EVENT_BIT_VAR_PREFIX + EVENT_REQUEST_PREFIX + event;
        }
        assert false;
        return null;
    }
    
    /**
     * Converts a state identification to String representing its Promela
     * label.
     */
    private String stateToLabel(int state) {
        return STATE_LABEL_PREFIX + state;
    }
    
    /**
     * Transform the preprocessed (i.e. instantiated) protocols into minimal
     * deterministic automata.
     *
     * @param architecture A preprocessed architecture whose protocols should
     *                      be minimalized in-place
     */
    private void prepareAutomata(PreprocessedArchitecture architecture) {
        for (BPComponent component : architecture.getComponents()) {
            Logger.getLogger("EBPTransformation").info("Minimalization started.");
            component.getProtocolKeeper().performTransformationChain(
                    Logger.getLogger("EBPTransformation"),
                    et);
            Logger.getLogger("EBPTransformation").info("Minimalization finished successfully!");
        }
    }
    
    
    /**
     * The event table
     */
    IEventTable et;
    
}
